@@ -115,6 +115,7 @@ class TLSRecordLayer(object):
         #Handshake digests
         self._handshake_md5 = hashlib.md5()
         self._handshake_sha = hashlib.sha1()
+        self._handshake_sha256 = hashlib.sha256()
 
         #TLS Protocol Version
         self.version = (0,0) #read-only
@@ -373,7 +374,7 @@ class TLSRecordLayer(object):
 
         @rtype: str
         @return: The name of the TLS version used with this connection.
-        Either None, 'SSL 3.0', 'TLS 1.0', or 'TLS 1.1'.
+        Either None, 'SSL 3.0', 'TLS 1.0', 'TLS 1.1', or 'TLS 1.2'.
         """
         if self.version == (3,0):
             return "SSL 3.0"
@@ -381,6 +382,8 @@ class TLSRecordLayer(object):
             return "TLS 1.0"
         elif self.version == (3,2):
             return "TLS 1.1"
+        elif self.version == (3,3):
+            return "TLS 1.2"
         else:
             return None
         
@@ -555,6 +558,7 @@ class TLSRecordLayer(object):
         if contentType == ContentType.handshake:
             self._handshake_md5.update(compat26Str(b))
             self._handshake_sha.update(compat26Str(b))
+            self._handshake_sha256.update(compat26Str(b))
 
         #Calculate MAC
         if self._writeState.macContext:
@@ -565,7 +569,7 @@ class TLSRecordLayer(object):
             if self.version == (3,0):
                 mac.update( compatHMAC( bytearray([len(b)//256] )))
                 mac.update( compatHMAC( bytearray([len(b)%256] )))
-            elif self.version in ((3,1), (3,2)):
+            elif self.version in ((3,1), (3,2), (3,3)):
                 mac.update(compatHMAC( bytearray([self.version[0]] )))
                 mac.update(compatHMAC( bytearray([self.version[1]] )))
                 mac.update( compatHMAC( bytearray([len(b)//256] )))
@@ -583,7 +587,7 @@ class TLSRecordLayer(object):
             if self._writeState.encContext.isBlockCipher:
 
                 #Add TLS 1.1 fixed block
-                if self.version == (3,2):
+                if self.version >= (3,2):
                     b = self.fixedIVBlock + b
 
                 #Add padding: b = b+ (macBytes + paddingBytes)
@@ -777,6 +781,7 @@ class TLSRecordLayer(object):
                 #Update handshake hashes
                 self._handshake_md5.update(compat26Str(p.bytes))
                 self._handshake_sha.update(compat26Str(p.bytes))
+                self._handshake_sha256.update(compat26Str(p.bytes))
 
                 #Parse based on handshake type
                 if subType == HandshakeType.client_hello:
@@ -786,7 +791,7 @@ class TLSRecordLayer(object):
                 elif subType == HandshakeType.certificate:
                     yield Certificate(constructorType).parse(p)
                 elif subType == HandshakeType.certificate_request:
-                    yield CertificateRequest().parse(p)
+                    yield CertificateRequest(self.version).parse(p)
                 elif subType == HandshakeType.certificate_verify:
                     yield CertificateVerify().parse(p)
                 elif subType == HandshakeType.server_key_exchange:
@@ -958,7 +963,7 @@ class TLSRecordLayer(object):
                             "Encrypted data not a multiple of blocksize"):
                         yield result
                 b = self._readState.encContext.decrypt(b)
-                if self.version == (3,2): #For TLS 1.1, remove explicit IV
+                if self.version >= (3,2): #For TLS 1.1, remove explicit IV
                     b = b[self._readState.encContext.block_size : ]
 
                 #Check padding
@@ -970,7 +975,7 @@ class TLSRecordLayer(object):
                 else:
                     if self.version == (3,0):
                         totalPaddingLength = paddingLength+1
-                    elif self.version in ((3,1), (3,2)):
+                    elif self.version in ((3,1), (3,2), (3,3)):
                         totalPaddingLength = paddingLength+1
                         paddingBytes = b[-totalPaddingLength:-1]
                         for byte in paddingBytes:
@@ -1007,7 +1012,7 @@ class TLSRecordLayer(object):
                 if self.version == (3,0):
                     mac.update( compatHMAC(bytearray( [len(b)//256] ) ))
                     mac.update( compatHMAC(bytearray( [len(b)%256] ) ))
-                elif self.version in ((3,1), (3,2)):
+                elif self.version in ((3,1), (3,2), (3,3)):
                     mac.update(compatHMAC(bytearray( [self.version[0]] ) ))
                     mac.update(compatHMAC(bytearray( [self.version[1]] ) ))
                     mac.update(compatHMAC(bytearray( [len(b)//256] ) ))
@@ -1034,6 +1039,7 @@ class TLSRecordLayer(object):
         self._client = client
         self._handshake_md5 = hashlib.md5()
         self._handshake_sha = hashlib.sha1()
+        self._handshake_sha256 = hashlib.sha256()
         self._handshakeBuffer = []
         self.allegedSrpUsername = None
         self._refCount = 1
@@ -1066,13 +1072,16 @@ class TLSRecordLayer(object):
         if cipherSuite in CipherSuite.shaSuites:
             macLength = 20
             digestmod = hashlib.sha1        
+        elif cipherSuite in CipherSuite.sha256Suites:
+            macLength = 32
+            digestmod = hashlib.sha256
         elif cipherSuite in CipherSuite.md5Suites:
             macLength = 16
             digestmod = hashlib.md5
 
         if self.version == (3,0):
             createMACFunc = createMAC_SSL
-        elif self.version in ((3,1), (3,2)):
+        elif self.version in ((3,1), (3,2), (3,3)):
             createMACFunc = createHMAC
 
         outputLength = (macLength*2) + (keyLength*2) + (ivLength*2)
@@ -1087,6 +1096,11 @@ class TLSRecordLayer(object):
                            b"key expansion",
                            serverRandom + clientRandom,
                            outputLength)
+        elif self.version == (3,3):
+            keyBlock = PRF_1_2(masterSecret,
+                           b"key expansion",
+                           serverRandom + clientRandom,
+                           outputLength)
         else:
             raise AssertionError()
 
@@ -1119,7 +1133,7 @@ class TLSRecordLayer(object):
             self._pendingWriteState = serverPendingState
             self._pendingReadState = clientPendingState
 
-        if self.version == (3,2) and ivLength:
+        if self.version >= (3,2) and ivLength:
             #Choose fixedIVBlock for TLS 1.1 (this is encrypted with the CBC
             #residue to create the IV for each sent block)
             self.fixedIVBlock = getRandomBytes(ivLength)
